Splitter

Struct Splitter 

pub struct Splitter<T> { /* private fields */ }
Splitter.

This node receives a vector, split it into chunks of size chunk_size and send each chunk to the next node.

Implementations

Source

impl<T> Splitter<T>
where T: Send + 'static + Clone,

pub fn build(chunk_size: usize) -> impl InOut<Vec<T>, Vec<T>>

Creates a new splitter node.

Arguments
  • chunk_size - Number of elements for each chunk.
Examples

Given a stream of numbers, we create a pipeline with a splitter that create vectors of two elements each.

use ppl::{prelude::*, templates::misc::{SourceIter, Splitter, SinkVec, Aggregator}};

let vec = vec![1, 2, 3, 4, 5, 6, 7, 8];
let p = pipeline![
    SourceIter::build(vec.into_iter()),
    Aggregator::build(8), // We aggregate each element in a vector.
    Splitter::build(2), // We split the received vector in 4 sub-vector of size 2.
    SinkVec::build()
];
let mut res = p.start_and_wait_end().unwrap();
assert_eq!(res.len(), 4)
pub fn build_with_replicas( n_replicas: usize, chunk_size: usize, ) -> impl InOut<Vec<T>, Vec<T>>

Creates a new splitter node with ‘n_replicas’ replicas of the same node.

Arguments
  • n_replicas - Number of replicas.
  • chunk_size - Number of elements for each chunk.